﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Net;
using System.Windows.Forms;

namespace RoboRace
{
    public partial class Form1 : Form
    {
        private string server;
        private string port;

        private string ID;
        private char[][] mapa;
        private Point pozice;
        private Point smer;
        private string povoleneTahy;

        private Point aktualniPozice;
        private Point aktualniSmer;
        private bool nalezenPoklad;

        private const int velikostPole = 20;
        private int velikostPlatna;
        private Bitmap bitmapMapa;
        private Graphics graphicsMapa;
        private Bitmap bitmapRobot;
        private Graphics graphicsRobot;

        public Form1()
        {
            InitializeComponent();

            velikostPlatna = velikostPole * 21 + 1;

            bitmapMapa = new Bitmap(velikostPlatna, velikostPlatna);
            graphicsMapa = Graphics.FromImage(bitmapMapa);

            bitmapRobot = new Bitmap(velikostPlatna, velikostPlatna);
            graphicsRobot = Graphics.FromImage(bitmapRobot);

            graphicsMapa.Clear(Color.White);
            graphicsRobot.Clear(Color.Transparent);

            panel1.Width = velikostPlatna;
            panel1.Height = velikostPlatna;
        }

        /// <summary>
        /// zpracuje odpověď serveru
        /// </summary>
        /// <param name="stream"></param>
        /// <returns></returns>
        private bool readResponse(Stream stream)
        {
            StreamReader reader = new StreamReader(stream);

            ID = reader.ReadLine();

            try
            {
                int.Parse(ID);
            }
            catch
            {
                Cursor = Cursors.Default;
                if (ID.Contains("Vyhrali"))
                {
                    MessageBox.Show(ID, "Konec hry", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                else
                {
                    MessageBox.Show(ID, "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                
                return false;
            }

            mapa = new char[20][];

            for (int y = 0; y < 20; y++)
            {
                mapa[y] = reader.ReadLine().ToCharArray(0, 20);
            }

            string pom = reader.ReadLine();
            string[] parameters = pom.Split(new char[] { ' ' });
            pozice = new Point(int.Parse(parameters[0]) - 1, int.Parse(parameters[1]) - 1);

            switch (parameters[2][0])
            {
                case 'N':
                    smer = new Point(0, -1);
                    break;

                case 'R':
                    smer = new Point(1, 0);
                    break;

                case 'D':
                    smer = new Point(0, 1);
                    break;

                case 'L':
                    smer = new Point(-1, 0);
                    break;
            }

            povoleneTahy = reader.ReadLine();

            reader.Close();
            return true;
        }

        /// <summary>
        /// získá odpověď serveru
        /// </summary>
        /// <param name="parametry"></param>
        /// <returns></returns>
        private bool getResponse(string parametry)
        {
            try
            {
                WebRequest request = WebRequest.Create("http://" + server + ":" + port + "/" + parametry);
                HttpWebResponse response = (HttpWebResponse)request.GetResponse();

                if (!readResponse(response.GetResponseStream())) return false;
            }
            catch (WebException e)
            {
                Cursor = Cursors.Default;
                MessageBox.Show(e.Message, "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            return true;
        }

        /// <summary>
        /// začátek hry
        /// </summary>
        /// <returns></returns>
        private bool getResponse()
        {
            return getResponse("start");
        }

        /// <summary>
        /// další kolo
        /// </summary>
        /// <param name="kolo"></param>
        /// <param name="pohyby"></param>
        /// <returns></returns>
        private bool getResponse(string kolo, string pohyby)
        {
            return getResponse("kolo?id=" + kolo + "&pohyby=" + pohyby);
        }

        /// <summary>
        /// vykreslí mapu do paměti
        /// </summary>
        private void vykreslitMapu()
        {
            graphicsMapa.Clear(Color.White);

            Pen blackPen = new Pen(Color.Black);
            Brush blackBrush = new SolidBrush(Color.Black);

            Font font = new Font("Tahoma", 15);

            for (int i = 0; i <= 20; i++)
            {
                graphicsMapa.DrawLine(blackPen, i * 21, 0, i * 21, velikostPlatna);
                graphicsMapa.DrawLine(blackPen, 0, i * 21, velikostPlatna, i * 21);
            }

            for (int y = 0; y < 20; y++)
            {
                for (int x = 0; x < 20; x++)
                {
                    graphicsMapa.DrawString(mapa[y][x].ToString(), font, blackBrush, x * 21, y * 21);
                }
            }


        }

        /// <summary>
        /// vykreslí robota do paměti
        /// </summary>
        private void vykreslitRobota()
        {
            Pen red = new Pen(Color.Red);
            Point[] body = new Point[3];

            if (aktualniSmer.X == 0 && aktualniSmer.Y == 1)
            {
                body[0] = new Point(aktualniPozice.X * 21, aktualniPozice.Y * 21);
                body[1] = new Point(aktualniPozice.X * 21 + 20, aktualniPozice.Y * 21);
                body[2] = new Point(aktualniPozice.X * 21 + 10, aktualniPozice.Y * 21 + 20);
            }
            else if (aktualniSmer.X == 0 && aktualniSmer.Y == -1)
            {
                body[0] = new Point(aktualniPozice.X * 21 + 10, aktualniPozice.Y * 21);
                body[1] = new Point(aktualniPozice.X * 21, aktualniPozice.Y * 21 + 20);
                body[2] = new Point(aktualniPozice.X * 21 + 20, aktualniPozice.Y * 21 + 20);
            }
            else if (aktualniSmer.X == -1 && aktualniSmer.Y == 0)
            {
                body[0] = new Point(aktualniPozice.X * 21, aktualniPozice.Y * 21 + 10);
                body[1] = new Point(aktualniPozice.X * 21 + 20, aktualniPozice.Y * 21);
                body[2] = new Point(aktualniPozice.X * 21 + 20, aktualniPozice.Y * 21 + 20);
            }
            else
            {
                body[0] = new Point(aktualniPozice.X * 21, aktualniPozice.Y * 21);
                body[1] = new Point(aktualniPozice.X * 21, aktualniPozice.Y * 21 + 20);
                body[2] = new Point(aktualniPozice.X * 21 + 20, aktualniPozice.Y * 21 + 10);
            }

            graphicsRobot.Clear(Color.Transparent);
            graphicsRobot.DrawPolygon(red, body);
        }

        /// <summary>
        /// spuštění interaktivní hry
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonInteraktivni_Click(object sender, EventArgs e)
        {
            groupBox1.Enabled = false;
            Cursor = Cursors.WaitCursor;

            server = textBoxServer.Text;
            port = textBoxPort.Text;

            if (!getResponse()) return;
            vykreslitMapu();

            aktualniPozice = pozice;
            aktualniSmer = smer;
            vykreslitRobota();

            panel1.CreateGraphics().DrawImage(bitmapMapa, 0, 0);
            panel1.CreateGraphics().DrawImage(bitmapRobot, 0, 0);

            groupBox1.Enabled = true;
            Cursor = Cursors.Default;

            textBoxPovolene.Text = povoleneTahy;
        }

        private void panel1_Paint(object sender, PaintEventArgs e)
        {
            panel1.CreateGraphics().DrawImage(bitmapMapa, 0, 0);
            panel1.CreateGraphics().DrawImage(bitmapRobot, 0, 0);
        }

        /// <summary>
        /// zkusí se pohybovat rovně o daný počet políček
        /// </summary>
        /// <param name="delka"></param>
        /// <returns></returns>
        private bool pohybRovne(int delka)
        {
            for (int i = 1; i <= delka; i++)
            {
                try
                {
                    char z = mapa[aktualniPozice.Y + aktualniSmer.Y][aktualniPozice.X + aktualniSmer.X];
                    if (z != 'X')
                    {
                        aktualniPozice.X += aktualniSmer.X;
                        aktualniPozice.Y += aktualniSmer.Y;
                    }
                    if (z == '#')
                    {
                        MessageBox.Show("Robot spadl do diry", "Informace", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        return false;
                    }
                    if (z == '$')
                    {
                        nalezenPoklad = true;
                    }
                }
                catch
                {
                    MessageBox.Show("Robot je mimo mapu", "Informace", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    return false;
                }
            }
            return true;
        }

        /// <summary>
        /// zkusí se pohybovat vzad
        /// </summary>
        /// <returns></returns>
        private bool pohybVzad()
        {
            try
            {
                char z = mapa[aktualniPozice.Y - aktualniSmer.Y][aktualniPozice.X - aktualniSmer.X];
                if (z != 'X')
                {
                    aktualniPozice.X -= aktualniSmer.X;
                    aktualniPozice.Y -= aktualniSmer.Y;
                }
                if (z == '#')
                {
                    MessageBox.Show("Robot spadl do diry", "Informace", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    return false;
                }
                if (z == '$')
                {
                    nalezenPoklad = true;
                }
            }
            catch
            {
                MessageBox.Show("Robot je mimo mapu", "Informace", MessageBoxButtons.OK, MessageBoxIcon.Information);
                return false;
            }
            return true;
        }

        /// <summary>
        /// otočí vlevo
        /// </summary>
        private void otocitVlevo()
        {
            if (aktualniSmer.X == 0 && aktualniSmer.Y == 1)
            {
                aktualniSmer.X = 1;
                aktualniSmer.Y = 0;
            }
            else if (aktualniSmer.X == 0 && aktualniSmer.Y == -1)
            {
                aktualniSmer.X = -1;
                aktualniSmer.Y = 0;
            }
            else if (aktualniSmer.X == -1 && aktualniSmer.Y == 0)
            {
                aktualniSmer.X = 0;
                aktualniSmer.Y = 1;
            }
            else
            {
                aktualniSmer.X = 0;
                aktualniSmer.Y = -1;
            }
        }

        /// <summary>
        /// otočí vpravo
        /// </summary>
        private void otocitVpravo()
        {
            if (aktualniSmer.X == 0 && aktualniSmer.Y == 1)
            {
                aktualniSmer.X = -1;
                aktualniSmer.Y = 0;
            }
            else if (aktualniSmer.X == 0 && aktualniSmer.Y == -1)
            {
                aktualniSmer.X = 1;
                aktualniSmer.Y = 0;
            }
            else if (aktualniSmer.X == -1 && aktualniSmer.Y == 0)
            {
                aktualniSmer.X = 0;
                aktualniSmer.Y = -1;
            }
            else
            {
                aktualniSmer.X = 0;
                aktualniSmer.Y = 1;
            }
        }

        /// <summary>
        /// zpracuje posun pro 1 tah (písmeno)
        /// </summary>
        /// <param name="tah"></param>
        private bool zpracovatTah(char tah)
        {
            switch (tah)
            {
                case '1':
                    if (!pohybRovne(1)) return false;
                    break;

                case '2':
                    if (!pohybRovne(2)) return false;
                    break;

                case '3':
                    if (!pohybRovne(3)) return false;
                    break;

                case 'L':
                    otocitVlevo();
                    break;

                case 'R':
                    otocitVpravo();
                    break;

                case 'Z':
                    if (!pohybVzad()) return false;
                    break;
            }

            if (mapa[aktualniPozice.Y][aktualniPozice.X] == '^')
            {
                try
                {
                    char z = mapa[aktualniPozice.Y - 1][aktualniPozice.X];
                    if (z != 'X')
                    {
                        aktualniPozice.Y--;
                    }
                    if (z == '#')
                    {
                        MessageBox.Show("Robot spadl do diry", "Informace", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        return false;
                    }
                    if (z == '$')
                    {
                        nalezenPoklad = true;
                    }
                }
                catch
                {
                    MessageBox.Show("Robot je mimo mapu", "Informace", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    return false;
                }
            }
            else if (mapa[aktualniPozice.Y][aktualniPozice.X] == 'v')
            {
                try
                {
                    char z = mapa[aktualniPozice.Y + 1][aktualniPozice.X];
                    if (z != 'X')
                    {
                        aktualniPozice.Y++;
                    }
                    if (z == '#')
                    {
                        MessageBox.Show("Robot spadl do diry", "Informace", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        return false;
                    }
                    if (z == '$')
                    {
                        nalezenPoklad = true;
                    }
                }
                catch
                {
                    MessageBox.Show("Robot je mimo mapu", "Informace", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    return false;
                }
            }
            else if (mapa[aktualniPozice.Y][aktualniPozice.X] == '<')
            {
                try
                {
                    char z = mapa[aktualniPozice.Y][aktualniPozice.X - 1];
                    if (z != 'X')
                    {
                        aktualniPozice.X--;
                    }
                    if (z == '#')
                    {
                        MessageBox.Show("Robot spadl do diry", "Informace", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        return false;
                    }
                    if (z == '$')
                    {
                        nalezenPoklad = true;
                    }
                }
                catch
                {
                    MessageBox.Show("Robot je mimo mapu", "Informace", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    return false;
                }
            }
            else if (mapa[aktualniPozice.Y][aktualniPozice.X] == '>')
            {
                try
                {
                    char z = mapa[aktualniPozice.Y][aktualniPozice.X + 1];
                    if (z != 'X')
                    {
                        aktualniPozice.X++;
                    }
                    if (z == '#')
                    {
                        MessageBox.Show("Robot spadl do diry", "Informace", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        return false;
                    }
                    if (z == '$')
                    {
                        nalezenPoklad = true;
                    }
                }
                catch
                {
                    MessageBox.Show("Robot je mimo mapu", "Informace", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    return false;
                }
            }

            if (nalezenPoklad)
            {
                MessageBox.Show("Poklad nalezen", "Informace", MessageBoxButtons.OK, MessageBoxIcon.Information);
                return false;
            }

            return true;
        }

        /// <summary>
        /// odeslání odpovědi uživatele
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonOdeslat_Click(object sender, EventArgs e)
        {
            if (!nalezenPoklad && textBoxHracovy.TextLength < 5)
            {
                MessageBox.Show("Poklad nenalezen, musíte zadat 5 tahů", "Informace", MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }

            groupBox1.Enabled = false;
            Cursor = Cursors.WaitCursor;
            if (!getResponse(ID, textBoxHracovy.Text))
            {
                groupBox1.Enabled = false;
                return;
            }

            aktualniPozice = pozice;
            aktualniSmer = smer;
            vykreslitRobota();

            panel1.CreateGraphics().DrawImage(bitmapMapa, 0, 0);
            panel1.CreateGraphics().DrawImage(bitmapRobot, 0, 0);

            groupBox1.Enabled = true;
            Cursor = Cursors.Default;
            textBoxPovolene.Text = povoleneTahy;
            textBoxHracovy.Text = "";
        }

        /// <summary>
        /// mění pozici robota
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void textBoxHracovy_TextChanged(object sender, EventArgs e)
        {
            aktualniPozice = pozice;
            aktualniSmer = smer;

            List<char> volne = povoleneTahy.ToList<char>();
            bool ok = true;

            for (int i = 0; i < textBoxHracovy.TextLength; i++)
            {
                char pismeno = textBoxHracovy.Text.ToUpper()[i];
                if (volne.Contains(pismeno))
                {
                    if (!zpracovatTah(pismeno)) break;
                    volne.Remove(pismeno);
                }
                else
                {
                    ok = false;
                }
            }

            buttonOdeslat.Enabled = (textBoxHracovy.TextLength > 0) ? ok : false;

            vykreslitRobota();

            panel1.CreateGraphics().DrawImage(bitmapMapa, 0, 0);
            panel1.CreateGraphics().DrawImage(bitmapRobot, 0, 0);
        }
    }
}